/**
 * LICENCE[[
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1/CeCILL 2.O
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is kelis.fr code.
 *
 * The Initial Developer of the Original Code is 
 * samuel.monsarrat@kelis.fr
 *
 * Portions created by the Initial Developer are Copyright (C) 2008-2012
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either of the GNU General Public License Version 2 or later (the "GPL"),
 * or the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * or the CeCILL Licence Version 2.0 (http://www.cecill.info/licences.en.html),
 * in which case the provisions of the GPL, the LGPL or the CeCILL are applicable
 * instead of those above. If you wish to allow use of your version of this file
 * only under the terms of either the GPL, the LGPL or the CeCILL, and not to allow
 * others to use your version of this file under the terms of the MPL, indicate
 * your decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL, the LGPL or the CeCILL. If you do not
 * delete the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL, the LGPL or the CeCILL.
 * ]]LICENCE
 */
/* ====================== SCENARI preload service ==============================
   This service preloads web pages in iframes before showing them thus resulting
   in a smoother experience.
   Prerequisites : 
    * The web generator's uiFrame must use a transparentIframe rootContext,
    * scPaLib.js must be loaded,
    * All links that open new pages must be well defined and their DOM scPaLib
      path properly declared through addNavBtnPath().
*/
scServices.preloadSvc = scOnLoads[scOnLoads.length] = {

	fMainFraPath : "ide:mainFrame",
	fDebug : false,
	fBtnPaths : [],

	/* === Public ============================================================= */

	/** preloadSvc.onLoad : SCENARI onLoad system */
	onLoad : function() {
		this.xLog("onLoad");
		this.fCurFra = scPaLib.findNode(this.fMainFraPath);
		this.fFraId = this.fCurFra.id;
		this.xInitFra(this.fCurFra);
	},

	/** preloadSvc.addNavBtnPath : add a scPaLib nav button path */
	addNavBtnPath : function (pPath){
		this.fBtnPaths.push(pPath);
	},

	/** preloadSvc.setMainFraPath : set the scPaLib path to the base iframe */
	setMainFraPath : function (pPath){
		this.fMainFraPath = pPath;
	},

	/** preloadSvc.goTo : preload and open a given url */
	goTo : function (pUrl){
		this.xLog("goTo: "+pUrl);
		if (!pUrl || pUrl.length==0) return;
		if (this.fCurFra.fNavStarted) return;
		this.fCurFra.fNavStarted = true;
		this.fNxtFra = this.xAddFra();
		this.fNxtFra.src = pUrl;
		return false;
	},

	/** preloadSvc.reload : reload the current page */
	reload : function (){
		this.goTo(this.fCurFra.src);
	},

	/* === Private ============================================================ */

	/** preloadSvc.sOnLoadPage : onload : this == iframe */
	sOnLoadPage : function (){
		try{
			if(scCoLib.isIE && this.readyState != "complete") return;
			scServices.preloadSvc.xInitPage(this);
		} catch(e){scServices.preloadSvc.xLog("ERROR error loading page: "+e)}
	},

	/** preloadSvc.xInitPage : process the current page init nav buttons etc */
	xInitPage : function (pFra){
		var vPgeDoc = pFra.contentWindow.document;
		scServices.preloadSvc.xLog("xInitPage: "+vPgeDoc.title);
		var vSvc = scServices.preloadSvc;
		if (vSvc.fNxtFra) {
			vSvc.fNxtFra.style.visibility="";
			vSvc.fCurFra.parentNode.removeChild(vSvc.fCurFra);
			vSvc.fCurFra = vSvc.fNxtFra;
			vSvc.fNxtFra = null;
			vSvc.fCurFra.id = vSvc.fFraId;
		}
		for(var i in vSvc.fBtnPaths) {
			var vBtns = scPaLib.findNodes(vSvc.fBtnPaths[i],vPgeDoc);
			for (var j in vBtns) vSvc.xInitBtn(vBtns[j]);
		}
		var vSubScDynUiMgr  = pFra.contentWindow.scDynUiMgr;
		if (vSubScDynUiMgr) vSubScDynUiMgr.subWindow.addOnLoadListener(vSvc.xInitPage);
	},

	/** preloadSvc.xInitBtn : init a given nav buttons */
	xInitBtn : function (pBtn){
		var vUrl = pBtn.href;
		if (pBtn.onclick) pBtn.funcOnclick = pBtn.onclick;
		pBtn.onclick = function() {try{if(this.funcOnclick) this.funcOnclick();}catch(e){} return scServices.preloadSvc.goTo(vUrl)};
	},

	/** preloadSvc.xAddFra : create a new iframe */
	xAddFra : function (){
		var vFra;
		if(scCoLib.isIE) {
			//BUG IE : impossible de masquer les bordures si on ajoute l'iframe via l'API DOM.
			var vFrmHolder = document.createElement("div");
			document.body.appendChild(vFrmHolder);
			vFrmHolder.innerHTML = "<iframe scrolling='no' frameborder='0'></iframe>";
			vFra = vFrmHolder.firstChild;
			document.body.appendChild(vFra);
			document.body.removeChild(vFrmHolder);
		} else {
			vFra = document.createElement("iframe");
			vFra.setAttribute("frameborder","0");
			document.body.appendChild(vFra);
		}
		vFra.style.width = "100%";
		vFra.style.height = "100%";
		vFra.style.visible = "hidden";
		this.xInitFra(vFra);
		return vFra;
	},

	/** preloadSvc.xInitFra : init a new iframe */
	xInitFra : function (pFra){
		if(scCoLib.isIE) pFra.onreadystatechange = scServices.preloadSvc.sOnLoadPage;
		else pFra.onload = scServices.preloadSvc.sOnLoadPage;
	},

	/** preloadSvc.xLog : log a messge to the console */
	xLog : function(pStr){
		try{
			if (this.fDebug) console.log("scServices.preloadSvc: "+pStr);
		} catch(e){}
	},
	loadSortKey : "zpreloadSvc"
}
